'The program COVERAGE TESTS perform the test for unconditional coverage and the test for conditional coverage (LRuc and LRcc respectively) for models that estimate VaR-values for returns. In order to run this program a work file needs to be open, with a return series r, a vector theta that contain the relevant VaR quantiles, a matrix var which contains corresponding VaR forecasts.

'-----------------------------------LRuc----------------------------------
vector(t) uc 'the vector that will contain the test statistic values of LRuc
vector(t) pe 'expected probabilities of violations, for each VaR-value
for !i = 1 to t
	if theta(!i) < 0.5 then
		pe(!i) = theta(!i)
	else
		pe(!i) = 1-theta(!i)
	endif
next

'lrhit equals 1 if exceedance and 0 otherwise
matrix(tf,t) lrhit
for !h = 1 to t
	for !i = 1 to tf
		if theta(!h) <0.5 then
			lrhit(!i,!h) = (r(ti+!i) < -var(!i,!h)) 'lrhit(.) = 1 if r(.)<-var(.) and 0 else
		else
			lrhit(!i,!h) = (r(ti+!i) > var(!i,!h)) 'lrhit(.) = 1 if r(.)>var(.) and 0 else
		endif
	next
next

vector(t) n1 = @csum(lrhit) 'number of observed exceedances
vector(t) n0  'number of returns that did not exceed the estimated VaR
vector(t) po 'observed probabilities of violations, for each VaR-value
for !i = 1 to t
	n0(!i) = tf-n1(!i)
	po(!i)= n1(!i)/tf
	if po(!i) = 1 or po(!i) = 0 then
		uc(!i) = "NA"
	else
		uc(!i) = -2*(n0(!i)*log(1-pe(!i))+n1(!i)*log(pe(!i))-n0(!i)*log(1-po(!i))-n1(!i)*log(po(!i)))
	endif
next

vector(t) ucpvalue
for !i = 1 to t
	if uc(!i) > 0 then
		ucpvalue(!i) = @chisq(uc(!i),1)
	else 'because of small errors in EViews' numerical calculations, uc  and cc can get negative results close to zero when they are supposed to be excactly zero, for instance when po = pe. Try for example to calculate 1-0.95, then EViews some times answers: 0.05000000000004.
		ucpvalue(!i) = 1
	endif
next
show ucpvalue
'---------------------------------LRcc-------------------------------------------------
vector(t) n11=0 'n11 is the number of times a violation is followed by another violation (i.e. when the lrhit-variable = 1 to times in a row
vector(t) n10=0 'n10 is the number of times a violation is followed by a non-violation (i.e. when the lrhit-variable is 1 for an observation and 0 for the next
vector(t) n01=0 'n01 is the number of times a non-violation is followed by a violation (i.e. when the lrhit-variable is 0 for an observation and 1 for the next
vector(t) n00=0 'n11 is the number of times a non-violation is followed by another non-violation (i.e. when the lrhit-variable = 0 to times in a row
for !h = 1 to t
	for !i = 1 to tf-1'this "for loop" calculates the n11,n10,n01 and n00. (but the last observation of "lrhit" is not included yet because of how the "if statements" are made. They will be taken care of afterwards.
		if lrhit(!i,!h) =1 then
			if lrhit(!i+1,!h) = 1 then
				n11(!h) = n11(!h)+1
			else
				n10(!h) = n10(!h)+1
			endif
		else
			if lrhit(!i+1,!h) = 1 then
				n01(!h) = n01(!h)+1
			else
				n00(!h) = n00(!h)+1
			endif
		endif
	next

'to make n0 = n00+n10 and n1 = n01+n11 the last value has to be included.
	if lrhit(tf,!h) = 1 then
		if n00(!h)+n10(!h) = n0(!h) then
			n11(!h) = n11(!h)+1
		else
			n10(!h) = n10(!h)+1
		endif
	else
		if n00(!h)+n10(!h) = n0(!h) then
			n01(!h) = n01(!h)+1
		else
			n00(!h) = n00(!h)+1
		endif
	endif
next

vector(t) p01 'the observed probability of the lrhit-variable going from 0 to 1
vector(t) p11 'the observed probability of the lrhit-variable going from 1 to 1
vector(t) cc 'The test statistic LRcc
for !i = 1 to t
	if n01(!i)=0 then
		p01(!i) = 0
	else
		p01(!i) = n01(!i)/(n00(!i)+n01(!i))
	endif

	if n11(!i)=0 then
		p11(!i) = 0
	else		
		p11(!i) = n11(!i)/(n10(!i)+n11(!i))
	endif

	'The LRcc test statistic cannot be calculated if the following statement holds, since log(0) is not defined.
	if p01(!i) = 1 or p01(!i) = 0 or p11(!i) = 0 or p11(!i)=1  then
		cc(!i) = "NA" 
	else
		cc(!i) = -2*(n0(!i)*log(1-pe(!i))+n1(!i)*log(pe(!i))-n00(!i)*log(1-p01(!i))-n01(!i)*log(p01(!i))-n10(!i)*log(1-p11(!i))-n11(!i)*log(p11(!i)))
	endif
next

vector(t) ccpvalue
for !i = 1 to t
	if cc(!i) > 0 then
		ccpvalue(!i) = @chisq(cc(!i),2)
	else
		ccpvalue(!i) = 1
	endif
next
show ccpvalue
