'The program EWQR computes VaR and ES forecasts using Exponentially Weigthed Quantile Regression (EQWR). In order to run the program, a workfile with a return series needs to be open; the return series must be named r. 

sample s1 @all 'initialize a sample s1
smpl s1
scalar tl = 250 'window size for lambda
scalar nr = @obs(r) 'number of returns
scalar tf = 500 'number of forecasts (out of sample window size)
scalar ti = nr - tf  'in-sample window size 
series b
series qr
series wqr
vector(6) theta 'replace '6' with the number of quantiles to consider, should it be something else
theta.fill 0.01,0.05,0.1,0.9,0.95,0.99 'create theta containing the relevant quantiles
scalar t = @rows(theta) 'number of quantiles

matrix(41,t+1) lam 'matrix where the first column will be 41 different values of lambda to test (from 0.8 to 1 with step size 0.005) , and the second,third etc. column will contain the corrisponding qrsums for theta = 0.01,0.05, etc.

for !k= 1 to 41
	lam(!k,1) = 0.8+0.005*(!k-1) 'lambda values from 0.8 to 1 with step size 0.005 assigned to first column of lam
	for !h = 1 to t 'make sure to do the calculations for each quantile
		for !i = 1 to ti-tl 'first, look at in-sample "forecasts" to optimize the choise of lambda
			series l
			for !j = 1 to tl
				l(!j+!i-1) = lam(!k,1)^(tl-!j) 'make a lambda vector to introduce as a weight in the quantile regression to do a EWQR
			next
			s1.set @first+!i-1 @first+!i+tl-2 'roll the sample s1 one step ahead
			smpl s1 'set s1 as the workfile sample
			equation eq01.qreg(quant=theta(!h),w=l,wscale=none) r c 'EWQR for the theta-th quantile with no regressors
			b(tl+!i) = c(1) 'save the quantile predictions
			delete l
		next
		smpl @all 'reset the sample to include all observations
		qr = (r-b)*(theta(!h)-((r-b)<0))
		lam(!k,!h+1) = @sum(qr) 'save the QRsum for a given combination of lambda and theta
	next
next

vector(t) minqrsum
vector(t) ol  'ol = the optimal lambda for each VaR-level

for !i = 1 to t
	minqrsum(!i) = lam(1,!i+1) 'initialize minqrsum to the first QRsum
	ol(!i) = lam(1,1) 'initialize ol to the first lambda (0.8)
next

for !i = 2 to 41 'find the lambdas that minimize the qrsums
	for !j = 1 to t
		if lam(!i,!j+1) < minqrsum(!j) then 
			minqrsum(!j) = lam(!i,!j+1)
			ol(!j) = lam(!i,1)
		endif
	next
next

'out-of-sample forcasts for the optimal lambdas
matrix(tf,t) es 'es = expected shorfall
matrix(tf,t) var 'var = value at risk
for !h = 1 to t
	for !i = 1 to tf  'out of sample forcasts
		series l
		for !j = 1 to tl
			l(!j+!i+ti-tl-1) = ol(!h)^(tl-!j) 'make a lambda vector  to introduce as a weight in the quantile regression to do a EWQR
		next
		s1.set @first+!i+ti-tl-1 @first+!i+ti-2 'roll the sample one step ahead
		smpl s1 'set s1 as the workfile sample
		equation eq01.qreg(quant=theta(!h),w=l,wscale=none) r c 'EWQR for the theta-th quantile with no regressors
		wqr = l*(r-c(1))*(theta(!h)-(r<c(1)))
		if theta(!h)<0.5 then
			es(!i,!h) = (1/(theta(!h)*@sum(l)))*@sum(wqr)
			var(!i,!h) = -c(1)
		else
			es(!i,!h) = (1/((1-theta(!h))*@sum(l)))*@sum(wqr)
			var(!i,!h) = c(1)
		endif
		delete l
	next
next
delete b eq01 qr s1 wqr
smpl @all
