#include "mex.h"
#include "math.h"

/**************************************************************************
 * C loop to compute Value at Risk for the Indirect AR(1)-GARCH(1,1) model.
 **************************************************************************
 * INPUT:
 *      BETA	= Regression Parameters
 *      r       = Return series
 *      VaR(1)  = Estimate of Value at Risk for the first observation in r
 **************************************************************************
 * OUTPUT:
 *      VaR     = Value at Risk Forecasts
 *************************************************************************/

/* subroutine */
void ComputeVaR(double THETA,double BETA[], double r[], double VarEstimate, double VaR[], int RowsOfr)
{
	int i;
	VaR[0] = VarEstimate; /* Initialize output variables */
	for(i = 1; i < RowsOfr; i++) /* Start the loop */
		{
        if (THETA>0.5)
            VaR[i] =  BETA[0]*r[i-1] + sqrt(BETA[1] + BETA[2]*pow((VaR[i-1]-BETA[0]*r[i-2]),2) + BETA[3]*pow((r[i-1]-BETA[0]*r[i-2]),2));
        else
            VaR[i] =  -BETA[0]*r[i-1] + sqrt(BETA[1] + BETA[2]*pow((VaR[i-1]+BETA[0]*r[i-2]),2) + BETA[3]*pow((r[i-1]-BETA[0]*r[i-2]),2));
        }
}/* end subroutine */

/* The gateway function */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	double *r, *VaR, *BETA;
	double THETA, VarEstimate;
	int RowsOfr;

	/* Check for proper number of arguments */
	if (nrhs != 4)
		mexErrMsgTxt("Four inputs required.");
	if (nlhs!=1)
		mexErrMsgTxt("One output required.");

	/* Get the scalar input THETA and VaR(1) */
	THETA             = mxGetScalar(prhs[0]);	
	VarEstimate = mxGetScalar(prhs[3]);
	
	/* Create a pointer to the input vectors BETA and r */
	BETA = mxGetPr(prhs[1]);
	r = mxGetPr(prhs[2]);

	/* Get the dimension of r */
	RowsOfr = mxGetM(prhs[2]);

	/* Set the output pointer */
	plhs[0] = mxCreateDoubleMatrix(RowsOfr, 1, mxREAL);

	/* Create a C pointer to a copy of the output vectors */
	VaR = mxGetPr(plhs[0]);

	/* Call the C subroutine */
	ComputeVaR(THETA, BETA, r, VarEstimate, VaR, RowsOfr);
}
