#include "mex.h"
#include "math.h"

/***********************************************************************
 * C loop to compute Value at Risk for the Indirect GARCH(1,1) model.
 ***********************************************************************
 * INPUT:
 *      BETA	= Regression Parameters
 *      r       = Return series
 *      VaR(1)  = Estimate of Value at Risk for the first observation in r
 *************************************************************************
 * OUTPUT:
 *      VaR     = Value at Risk Forecasts
 ***********************************************************************/

/* subroutine */
void ComputeVaR(double BETA[], double r[], double VarEstimate, double VaR[], int RowsOfr)
{
	int i;
	VaR[0] = VarEstimate; /* Initialize output variables */
	for(i = 1; i < RowsOfr; i++) /* Start the loop */
		{
        VaR[i] =  sqrt(BETA[0] + BETA[1]*pow(VaR[i-1],2) + BETA[2]*pow(r[i-1],2));
        }
}/* end subroutine */

/* The gateway function */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	double *r, *VaR, *BETA;
	double VarEstimate;
	int RowsOfr;

	/* Check for proper number of arguments */
	if (nrhs != 3)
		mexErrMsgTxt("Three inputs required.");
	if (nlhs!=1)
		mexErrMsgTxt("One output required.");

	/* Get the scalar input VaR(1) */
	VarEstimate = mxGetScalar(prhs[2]);
	
	/* Create a pointer to the input vectors BETA and r */
	BETA = mxGetPr(prhs[0]);
	r = mxGetPr(prhs[1]);

	/* Get the dimension of r */
	RowsOfr = mxGetM(prhs[1]);

	/* Set the output pointer */
	plhs[0] = mxCreateDoubleMatrix(RowsOfr, 1, mxREAL);

	/* Create a C pointer to a copy of the output vectors */
	VaR = mxGetPr(plhs[0]);

	/* Call the C subroutine */
	ComputeVaR(BETA, r, VarEstimate, VaR, RowsOfr);
}
