function [Ht,weights] = riskmetrics2006(data,tau0,tau1,kmax,rho)
% EXAMPLES
%   RiskMetrics 2006 methodology using the suggested reference values
%     tau0   = 1560;
%     tau1   = 4;
%     taumax = 512;
%     rho    = sqrt(2);
%     kmax   = round(log(taumax/tau1)/log(rho))
%     Ht     = riskmetrics2006(data,tau0,tau1,kmax,rho)
%
%   RiskMetrics 1994 methodology using .94 as a special case of RM2006
%     tau0 = 1560;         % Does not matter
%     rho  = 1;             % Does not matter
%     tau1 = -1/log(.94);
%     kmax = 1;
%     Ht = riskmetrics2006(data,tau0,tau1,kmax,rho)



[T,K] = size(data);
backCast = cov(data); % Should not do this
tauks = tau1*rho.^((1:kmax)-1);
w = 1-log(tauks)/log(tau0);
w=w/sum(w);
Ht = zeros(K,K,T);
Httilde = zeros(K,K,T);
Httilde(:,:,1) = backCast;
for k=1:kmax
    tauk = tauks(k);
    mu = exp(-1/tauk);
    for t=2:T
        Httilde(:,:,t) = mu*Httilde(:,:,t-1) + (1-mu)*data(t-1,:)'*data(t-1,:);
    end
    Ht = Ht + w(k) * Httilde;
end


if nargout>1
    weights = zeros(T,T);
    for k=1:kmax
        tauk = tauks(k);
        mu = exp(-1/tauk);
        weightMatrix = [mu.^(0:T-1)' zeros(T,T-1)];
        weightMatrix(T,T) = (1-mu);
        for j=1:(T-2);
            weightMatrix(T,T-j) = mu * weightMatrix(T,T-j+1);
        end
        for t=2:T-1
            weightMatrix(t,2:t) = weightMatrix(T,T+(-t+2:0));
        end
        weights = weights + w(k) * weightMatrix;
    end
end





