function [UCpvalue CCpvalue] = CoverageTests(T,THETA,H)

% CoverageTests computes the tests for unconditional and conditional 
% coverage
%
%INPUT:
%   T     = Number of forecasts
%   THETA = Value at Risk quantile
%   H     = 1 if r exceeds VaR, 0 otherwise
%
%OUTPUT:
%   UCpvalue = P-value for the unconditional coverage test
%   CCpvalue = P-value for the conditional coverage test


%Test for unconditional coverage
n1 = sum(H);
n0 = T-n1;
po = n1/T;

if THETA > 0.5
    pe = 1- THETA;
else
    pe = THETA;
end

if po == 1 || po == 0 %avoid an expression with log(0)
    UCpvalue = NaN;
else
    UCstat = -2*(n0*log(1-pe)+n1*log(pe)-n0*log(1-po)-n1*log(po));
    UCpvalue = 1-chi2cdf(UCstat,1);
end

%Test for conditional coverage
n11 = 0;
n10 = 0;
n01 = 0;
n00 = 0;
%calculate  n11,n10,n01 and n00.
for i = 1:T-1 
    if H(i) == 1
        if H(i+1) == 1
            n11 = n11+1;
        else
            n10 = n10+1;
        end
    elseif H(i+1) == 1
        n01 = n01+1;
    else
        n00 = n00+1;
    end   
end

%Include last observation to ensure that n0 = n00+n10 and n1 = n01+n11
if H(T) == 1
    if n00 + n10 == n0,
        n11 = n11+1;
    else
       n10 = n10+1;
    end
elseif n00+n10 == n0
    n01 = n01+1;
else
	n00 = n00+1;
end

%The following two if-loops avoid division by zero
if n01 == 0
    p01 = 0;
else
    p01 = n01/(n00+n01);
end

if n11 == 0
    p11 = 0;
else
    p11 = n11/(n10+n11);
end


if p01 == 1 || p01 == 0 || p11 == 0 || p11 == 1 %avoid an expression with log(0)
    CCpvalue = NaN;
else
	CCstat = -2*(n0*log(1-pe)+n1*log(pe)-n00*log(1-p01)-n01*log(p01)-n10*log(1-p11)-n11*log(p11));
    CCpvalue = 1-chi2cdf(CCstat,2);
end

    