function DQpvalue = DQtest(T, THETA, VaR, H)

% DQTEST computes the Dynamic Quantile (DQ) test
%
%INPUT:
%   T     = Number of forecasts
% 	THETA = Value at Risk quantile
%   VaR   = Value at Risk forecasts
%   H     = 1 if r exceeds VaR, 0 otherwise
%
%OUTPUT:
%   DQpvalue = P-value for the Dynamic Quantile test

LAGS = 4; % Number of lagged hits to be included in the test.

% Compute the regressor matrix X.
Constant = ones(T-LAGS,1);
VaRforecast = VaR(LAGS+1:T);
Z = zeros(T-LAGS,LAGS);

if THETA > 0.5
    Hit = H - 1 + THETA;
else
    Hit = H - THETA;
end

for s = 1:LAGS
   Z(:,s) = Hit(s:T-(LAGS+1-s));
end

X = [Constant, VaRforecast, Z];
HIT = Hit(LAGS+1:T);

% Compute the DQ test statistic and the corresponding p-value.
DQstat = (HIT'*X*inv(X'*X)*X'*HIT)/(THETA*(1-THETA));
DQpvalue = 1-chi2cdf(DQstat,size(X,2));
